"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFrontendContent = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const ts_poet_1 = require("ts-poet");
const utils_1 = require("./utils");
const ClientProviderOptions = ts_poet_1.imp('ClientProviderOptions@@nestjs/microservices');
const Transport = ts_poet_1.imp('Transport@@nestjs/microservices');
function generateFrontendService(service, srcProtosDir, fileDescriptorProto, serviceDescriptorProto, typeMap) {
    utils_1.assertDefined(fileDescriptorProto.name);
    fs_1.copyFileSync(path_1.join(srcProtosDir, fileDescriptorProto.name), path_1.join(service.protosDir, fileDescriptorProto.name));
    const methodDefinitions = serviceDescriptorProto.method
        .map(method => utils_1.getMethodDefinition(method, typeMap))
        .reduce(utils_1.combineCode, ts_poet_1.code ``);
    return ts_poet_1.code `
    export const ${serviceDescriptorProto.name}ClientProviderOptions: ${ClientProviderOptions} = {
      name: '${serviceDescriptorProto.name}',
      transport: ${Transport}.GRPC,
      options: {
        package: '${fileDescriptorProto.package}',
        protoPath: '../protos/${fileDescriptorProto.name}'
      }
    };

    export interface ${serviceDescriptorProto.name}Client {
      ${methodDefinitions}
    }
  `;
}
async function generateFrontendContent(service, protosDir, fileDescriptorProto, typeMap) {
    const codeContent = fileDescriptorProto.service
        .map(serviceDescriptorProto => generateFrontendService(service, protosDir, fileDescriptorProto, serviceDescriptorProto, typeMap))
        .reduce(utils_1.combineCode, ts_poet_1.code ``);
    return utils_1.createGeneratedFile(service, fileDescriptorProto, 'frontend', codeContent);
}
exports.generateFrontendContent = generateFrontendContent;
