"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertDefined = exports.getMethodDefinition = exports.combineCode = exports.createGeneratedFileForBackendMicroserviceOptions = exports.createGeneratedFile = void 0;
const path_1 = require("path");
const protobufjs_1 = require("protobufjs");
const ts_poet_1 = require("ts-poet");
const types_1 = require("./types");
var normalize = protobufjs_1.util.path.normalize;
const Observable = ts_poet_1.imp('Observable@rxjs');
function prefixDisableLinter(fileContents) {
    return `/* eslint-disable */
${fileContents}`;
}
async function createGeneratedFile(service, fileDescriptorProto, type, codeContent) {
    assertDefined(fileDescriptorProto.package);
    assertDefined(fileDescriptorProto.name);
    const directory = fileDescriptorProto.package.replace('.', '/');
    const fileName = `${fileDescriptorProto.name.replace('.proto', '')}.${type}.ts`;
    const relativePath = normalize(path_1.join(directory, fileName));
    const fullPath = normalize(path_1.join(service.generatedDir, relativePath));
    const content = prefixDisableLinter(await codeContent.toStringWithImports({ path: relativePath }));
    return {
        getFilename: () => fullPath,
        getContent: () => content,
    };
}
exports.createGeneratedFile = createGeneratedFile;
async function createGeneratedFileForBackendMicroserviceOptions(service, codeContent) {
    const fullPath = normalize(path_1.join(service.generatedDir, 'backend-microservice-options.ts'));
    const content = prefixDisableLinter(await codeContent.toStringWithImports());
    return {
        getFilename: () => fullPath,
        getContent: () => content,
    };
}
exports.createGeneratedFileForBackendMicroserviceOptions = createGeneratedFileForBackendMicroserviceOptions;
function combineCode(acc, cur) {
    return ts_poet_1.code `${acc}\n${cur}`;
}
exports.combineCode = combineCode;
function getInputType(inputType, typeMap) {
    return types_1.getImpFromTypeName(typeMap, inputType);
}
function getNormalOutputType(outputType, typeMap) {
    return types_1.getImpFromTypeName(typeMap, outputType);
}
function getPromiseOutputType(outputType, typeMap) {
    return ts_poet_1.code `Promise<${types_1.getImpFromTypeName(typeMap, outputType)}>`;
}
function getObservableOutputType(outputType, typeMap) {
    return ts_poet_1.code `${Observable}<${types_1.getImpFromTypeName(typeMap, outputType)}>`;
}
function getOutputType(outputType, typeMap) {
    return ts_poet_1.code `${getNormalOutputType(outputType, typeMap)}
                | ${getPromiseOutputType(outputType, typeMap)}
                | ${getObservableOutputType(outputType, typeMap)}`;
}
function getMethodDefinition({ name, inputType, outputType }, typeMap) {
    assertDefined(inputType);
    assertDefined(outputType);
    return ts_poet_1.code `${name}(request: ${getInputType(inputType, typeMap)}): ${getOutputType(outputType, typeMap)}`;
}
exports.getMethodDefinition = getMethodDefinition;
function assertDefined(value) {
    if (value === undefined || value === null) {
        throw new Error(`Variable was not defined when it should be`);
    }
}
exports.assertDefined = assertDefined;
