import { copyFileSync } from 'fs';
import { join } from 'path';
import { code, imp } from 'ts-poet';
import { assertDefined, combineCode, createGeneratedFile, getMethodDefinition } from './utils';
const ClientProviderOptions = imp('ClientProviderOptions@@nestjs/microservices');
const Transport = imp('Transport@@nestjs/microservices');
function generateFrontendService(service, srcProtosDir, fileDescriptorProto, serviceDescriptorProto, typeMap) {
    assertDefined(fileDescriptorProto.name);
    copyFileSync(join(srcProtosDir, fileDescriptorProto.name), join(service.protosDir, fileDescriptorProto.name));
    const methodDefinitions = serviceDescriptorProto.method
        .map(method => getMethodDefinition(method, typeMap))
        .reduce(combineCode, code ``);
    return code `
    export const ${serviceDescriptorProto.name}ClientProviderOptions: ${ClientProviderOptions} = {
      name: '${serviceDescriptorProto.name}',
      transport: ${Transport}.GRPC,
      options: {
        package: '${fileDescriptorProto.package}',
        protoPath: '../protos/${fileDescriptorProto.name}'
      }
    };

    export interface ${serviceDescriptorProto.name}Client {
      ${methodDefinitions}
    }
  `;
}
export async function generateFrontendContent(service, protosDir, fileDescriptorProto, typeMap) {
    const codeContent = fileDescriptorProto.service
        .map(serviceDescriptorProto => generateFrontendService(service, protosDir, fileDescriptorProto, serviceDescriptorProto, typeMap))
        .reduce(combineCode, code ``);
    return createGeneratedFile(service, fileDescriptorProto, 'frontend', codeContent);
}
