import { join } from 'path';
import { util } from 'protobufjs';
import { code, imp } from 'ts-poet';
import { getImpFromTypeName } from './types';
var normalize = util.path.normalize;
const Observable = imp('Observable@rxjs');
function prefixDisableLinter(fileContents) {
    return `/* eslint-disable */
${fileContents}`;
}
export async function createGeneratedFile(service, fileDescriptorProto, type, codeContent) {
    assertDefined(fileDescriptorProto.package);
    assertDefined(fileDescriptorProto.name);
    const directory = fileDescriptorProto.package.replace('.', '/');
    const fileName = `${fileDescriptorProto.name.replace('.proto', '')}.${type}.ts`;
    const relativePath = normalize(join(directory, fileName));
    const fullPath = normalize(join(service.generatedDir, relativePath));
    const content = prefixDisableLinter(await codeContent.toStringWithImports({ path: relativePath }));
    return {
        getFilename: () => fullPath,
        getContent: () => content,
    };
}
export async function createGeneratedFileForBackendMicroserviceOptions(service, codeContent) {
    const fullPath = normalize(join(service.generatedDir, 'backend-microservice-options.ts'));
    const content = prefixDisableLinter(await codeContent.toStringWithImports());
    return {
        getFilename: () => fullPath,
        getContent: () => content,
    };
}
export function combineCode(acc, cur) {
    return code `${acc}\n${cur}`;
}
function getInputType(inputType, typeMap) {
    return getImpFromTypeName(typeMap, inputType);
}
function getNormalOutputType(outputType, typeMap) {
    return getImpFromTypeName(typeMap, outputType);
}
function getPromiseOutputType(outputType, typeMap) {
    return code `Promise<${getImpFromTypeName(typeMap, outputType)}>`;
}
function getObservableOutputType(outputType, typeMap) {
    return code `${Observable}<${getImpFromTypeName(typeMap, outputType)}>`;
}
function getOutputType(outputType, typeMap) {
    return code `${getNormalOutputType(outputType, typeMap)}
                | ${getPromiseOutputType(outputType, typeMap)}
                | ${getObservableOutputType(outputType, typeMap)}`;
}
export function getMethodDefinition({ name, inputType, outputType }, typeMap) {
    assertDefined(inputType);
    assertDefined(outputType);
    return code `${name}(request: ${getInputType(inputType, typeMap)}): ${getOutputType(outputType, typeMap)}`;
}
export function assertDefined(value) {
    if (value === undefined || value === null) {
        throw new Error(`Variable was not defined when it should be`);
    }
}
